#include <cyg/kernel/kapi.h>
#include "kbd.h"

cyg_flag_t kbd_flag;

static cyg_interrupt keyboard_interrupt;
static cyg_handle_t keyboard_handle;

#define	KBDATAPORT	0x0060		/* Data I/O port */
#define	KBCMDPORT	0x0064		/* Command port (write) */
#define	KBSTATPORT	0x0064		/* Status port	(read) */

static int kbd_code;

/*
 *-----------------------------------------------------------------------------
 *
 * keyboard_isr --
 *
 * 	The ISR to handle keyboard events.
 *
 * Results:
 *	Returns a flag indicating that the DSR should be run.
 *
 * Side Effects:
 *
 *	Sets the global variable kbd_code to the scan code, or 0 if
 *	the keyboard was not readable.
 *
 *	Masks the interrupt so that further interrupts will not be
 *	received until the DSR is run.
 *
 *-----------------------------------------------------------------------------
 */

static cyg_uint32 keyboard_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    CYG_BYTE stat, code;
    CYG_BYTE c;

    cyg_interrupt_mask(vector);
    cyg_interrupt_acknowledge(vector);

    kbd_code = 0;
    HAL_READ_UINT8( KBSTATPORT, stat );

    /* Keyboard is not ready to be read. */
    if((stat & 0x01) == 0)
	kbd_code = 0;

    /* Read the keyboard scan code. */
    HAL_READ_UINT8( KBDATAPORT, kbd_code );

    return (CYG_ISR_HANDLED | CYG_ISR_CALL_DSR);
}

/*
 *-----------------------------------------------------------------------------
 *
 * keyboard_dsr --
 *
 * 	"Top half" of the keyboard interrupt handling.  Performs
 * 	different actions depending on the key pressed.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *
 *	Depending on the key pressed, sets the 'kbd_flag' to different
 *	values.  This flag is collected in the main loop routine in
 *	gfx.c.
 *
 *-----------------------------------------------------------------------------
 */

static void keyboard_dsr(cyg_vector_t vector, cyg_ucount32 count,
		    cyg_addrword_t data)
{
    switch (kbd_code) {
    case 0x39: /* space key */
	cyg_flag_setbits(&kbd_flag, SPACE_FLAG);
	break;
    case 0x21: /* 'f' */
	cyg_flag_setbits(&kbd_flag, FASTER_FLAG);
	break;
    case 0x1f: /* 's' */
	cyg_flag_setbits(&kbd_flag, SLOWER_FLAG);
	break;
    default:
	break;
    }

    cyg_interrupt_unmask(vector);
}

/*
 *-----------------------------------------------------------------------------
 *
 * setup_kbd --
 *
 * 	Set up the keyboard interrupt handling code.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *
 *	Sets up the interrupt handler for the PC keyboard, and unmasks
 *	it, meaning that the code should be ready to handle keypresses
 *	when this is called.
 *
 *-----------------------------------------------------------------------------
 */

void setup_kbd()
{
    cyg_vector_t keyboard_vector = CYGNUM_HAL_INTERRUPT_KEYBOARD;
    cyg_priority_t keyboard_priority = 1;

    cyg_interrupt_create(
	keyboard_vector,
	keyboard_priority,
	0,
	&keyboard_isr,
	&keyboard_dsr,
	&keyboard_handle,
	&keyboard_interrupt);

    cyg_flag_init(&kbd_flag);

    cyg_interrupt_attach(keyboard_handle);
    cyg_interrupt_unmask(keyboard_vector);
}
